# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingBodyTemperatureSampleSensorLocation(str, enum.Enum):
    ARMPIT = "armpit"
    BODY = "body"
    EAR = "ear"
    FINGER = "finger"
    GASTROINTESTINAL = "gastrointestinal"
    MOUTH = "mouth"
    RECTUM = "rectum"
    TOE = "toe"
    EARDRUM = "eardrum"
    TEMPORAL_ARTERY = "temporal_artery"
    FOREHEAD = "forehead"
    WRIST = "wrist"
    _UNKNOWN = "__CLIENTFACINGBODYTEMPERATURESAMPLESENSORLOCATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ClientFacingBodyTemperatureSampleSensorLocation":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        armpit: typing.Callable[[], T_Result],
        body: typing.Callable[[], T_Result],
        ear: typing.Callable[[], T_Result],
        finger: typing.Callable[[], T_Result],
        gastrointestinal: typing.Callable[[], T_Result],
        mouth: typing.Callable[[], T_Result],
        rectum: typing.Callable[[], T_Result],
        toe: typing.Callable[[], T_Result],
        eardrum: typing.Callable[[], T_Result],
        temporal_artery: typing.Callable[[], T_Result],
        forehead: typing.Callable[[], T_Result],
        wrist: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ClientFacingBodyTemperatureSampleSensorLocation.ARMPIT:
            return armpit()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.BODY:
            return body()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.EAR:
            return ear()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.FINGER:
            return finger()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.GASTROINTESTINAL:
            return gastrointestinal()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.MOUTH:
            return mouth()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.RECTUM:
            return rectum()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.TOE:
            return toe()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.EARDRUM:
            return eardrum()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.TEMPORAL_ARTERY:
            return temporal_artery()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.FOREHEAD:
            return forehead()
        if self is ClientFacingBodyTemperatureSampleSensorLocation.WRIST:
            return wrist()
        return _unknown_member(self._value_)
