# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_destination_preferences_enabled_item import EventDestinationPreferencesEnabledItem
from .event_destination_preferences_preferred import EventDestinationPreferencesPreferred


class EventDestinationPreferences(UniversalBaseModel):
    preferred: EventDestinationPreferencesPreferred = pydantic.Field()
    """
    ℹ️ This enum is non-exhaustive.
    """

    enabled: typing.List[EventDestinationPreferencesEnabledItem]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
