# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingStandHourSampleType(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    STOOD = "stood"
    IDLE = "idle"
    _UNKNOWN = "__CLIENTFACINGSTANDHOURSAMPLETYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ClientFacingStandHourSampleType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        stood: typing.Callable[[], T_Result],
        idle: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ClientFacingStandHourSampleType.STOOD:
            return stood()
        if self is ClientFacingStandHourSampleType.IDLE:
            return idle()
        return _unknown_member(self._value_)
